/*----------------------------------------------------------------------------*/
/* Copyright (c) 2015-2018 FIRST. All Rights Reserved.                        */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

#include "NetworkListener.h"

using namespace cs;

class NetworkListener::Impl {};

NetworkListener::NetworkListener(wpi::Logger& logger, Notifier& notifier) {}

NetworkListener::~NetworkListener() {}

void NetworkListener::Start() {}

void NetworkListener::Stop() {}
