/*----------------------------------------------------------------------------*/
/* Copyright (c) 2016-2018 FIRST. All Rights Reserved.                        */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

#ifndef CSCORE_CSCORE_H_
#define CSCORE_CSCORE_H_

/* C API */
#include "cscore_c.h"

#ifdef __cplusplus
/* C++ API */
#include "cscore_cpp.h"
#include "cscore_oo.h"
#endif /* __cplusplus */

#endif  // CSCORE_CSCORE_H_
